@@ -3,22 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
  * 
- * This file contains Original Code and/or Modifications of Original Code
- * as defined in and that are subject to the Apple Public Source License
- * Version 2.0 (the 'License'). You may not use this file except in
- * compliance with the License. Please obtain a copy of the License at
- * http://www.opensource.apple.com/apsl/ and read it before using this
- * file.
- * 
- * The Original Code and all software distributed under the License are
- * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
- * Please see the License for the specific language governing rights and
- * limitations under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -38,8 +35,6 @@
 #define super IOPlatformPIDCtrlLoop
 OSDefineMetaClassAndStructors(PowerMac7_2_CPUFanCtrlLoop, IOPlatformPIDCtrlLoop)
 
-//extern const OSSymbol * gPM72EnvShroudRemoved;
-//extern const OSSymbol * gPM72EnvAllowNapping;
 extern PowerMac7_2_PlatformPlugin * PM72Plugin;
 extern const OSSymbol * gPM72EnvSystemUncalibrated;
 
@@ -177,7 +172,7 @@ bool PowerMac7_2_CPUFanCtrlLoop::choosePIDDataset( const OSDictionary * ctrlLoop
 	int i, count, cmp;
 	const OSArray * tmp_array;
 	OSArray * metaStateArray;
-	OSDictionary * eepromDataset, * plistDataset, * chosenDataset;
+	OSDictionary * eepromDataset, * plistDataset = NULL, * chosenDataset;
 	const OSData * binID, * plistBinID;
 
 	if (!ctrlLoopDict) return(false);
@@ -1070,18 +1065,6 @@ void PowerMac7_2_CPUFanCtrlLoop::sendNewTarget( ControlValue newTarget )
 {
 	ControlValue secNewTarget;
 
-/*
-	// If the fans are at minimum speed, don't let the processors nap.  We need to
-	// keep the heat sink warm or it loses efficiency.
-	bool allowNapping = (newTarget->unsigned32BitValue() != outputMin);
-
-	// Sync the internal napping state with the platform plugin's environment dict
-	if (ctrlloopState == kIOPCtrlLoopFirstAdjustment ||
-	    allowNapping != platformPlugin->envArrayCondIsTrueForObject(this, gPM72EnvAllowNapping))
-	{
-		platformPlugin->setEnvArray(gPM72EnvAllowNapping, this, allowNapping);
-	}
-*/
 	// If the new target value is different, send it to the control
 	if (ctrlloopState == kIOPCtrlLoopFirstAdjustment ||
 	    ctrlloopState == kIOPCtrlLoopDidWake ||
